import datetime
import importlib
import logging
import os
import re
import sys
from datetime import timezone
from urllib.parse import urlparse

import dateutil.parser
import gi

gi.require_version("Gst", "1.0")
gi.require_version("GstPbutils", "1.0")
from gi.repository import Gio, GLib, Gst, GstPbutils

from . import __version__
from .config import Config
from .options import Options
from .schema import (
    Audio,
    Container,
    Field,
    File,
    Info,
    Meta,
    Schema,
    Stream,
    Subtitle,
    Tag,
    Video,
)

logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
logger = logging.getLogger(__name__)


class DiscovererOptions(Options):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        group = self.add_argument_group("discovery")
        group.add_argument(
            "-q", "--quit", action="store_true", default=False, help="Quit after scan"
        )
        group.add_argument(
            "-d", "--dir", action="store", help="media content directory"
        )
        group.add_argument(
            "-f", "--force", action="store_true", default=False, help="force rescanning"
        )


class Discoverer(object):
    def __init__(self, config, args):
        Gst.init(None)

        self.config = config
        self.loop = GLib.MainLoop()
        self.quit = args.quit
        self.dir = args.dir
        self.force = args.force
        schema = Schema(config)
        self.session = schema.create_session()
        # TODO Set the logging level
        # Write down the Meta table
        meta = self.session.query(Meta).one_or_none()
        if not meta:
            meta = Meta()
            meta.version = __version__
            meta.root = self.dir
            self.session.add(meta)
        else:
            meta.version = __version__
        self.session.commit()
        # TODO Check in case we have provided a different folder
        # Start analyzing the provided media path
        self.path = Gio.File.new_for_path(self.dir)
        # Add a monitor
        self.monitor = self.path.monitor_directory(0, None)
        self.monitor.connect("changed", self.on_changed)
        self.discoverer = GstPbutils.Discoverer.new(5 * Gst.SECOND)
        self.discoverer.connect("discovered", self.on_discovered)
        self.discoverer.connect("finished", self.on_finished)
        self.numdiscoveries = 0
        self.discoverer.start()
        # Iterate over the files in the directory
        self.numdirs = 1
        self.path.enumerate_children_async(
            "*",
            Gio.FileQueryInfoFlags.NOFOLLOW_SYMLINKS,
            0,
            None,
            self.on_directory_content,
            None,
        )

    def _has_gst_override(self):
        # FIXME we need python3-gst-1.0 package to be installed in the virtualenv for
        # unknown types (overrides). Sadly there is no pip package for it.
        spec = importlib.util.find_spec("gi.overrides.Gst")
        if spec:
            return True
        else:
            return False

    def _parse_type_value(self, s):
        taglist = re.sub("(\{(?:\[??[^\[]*?\}))", "", s)
        fields = taglist.split(",")
        if len(fields) > 1:
            regex = r"(?P<key>[\w-]*)\=\((?P<type>\w+)\)[\"]?(?P<value>[\w -\\]*)[\"]?"
            regex_c = re.compile(regex)
            # Remove trailing semicolon
            for field in fields[1:-1] + [fields[-1][:-1]]:
                groups = re.match(regex_c, field.strip())
                yield (groups.group("key"), groups.group("type"), groups.group("value"))

    def rel_path(self, path):
        rel_path = os.path.relpath(path, self.path.get_path())
        (dirname, basename) = os.path.split(rel_path)
        return (dirname, basename)

    def discover_file(self, path, mtime):
        (dirname, basename) = self.rel_path(path)
        (exists, needs_update) = self.file_stat(basename, dirname, mtime)
        if not exists:
            # Fill the File information
            self.add_file(basename, dirname, mtime)
        if needs_update or self.force:
            # Start discovering
            uri = "file://{}".format(path)
            logger.debug(
                "Discovering {} (numdiscoveries: {})".format(uri, self.numdiscoveries)
            )
            self.numdiscoveries += 1
            self.discoverer.discover_uri_async(uri)

    def on_changed(self, monitor, f, of, event_type):
        logger.debug(
            "A change of type {} happened in the media directory".format(event_type)
        )
        if event_type == Gio.FileMonitorEvent.CREATED:
            logger.debug("File {} created".format(f.get_path()))
            finfo = f.query_info(Gio.FILE_ATTRIBUTE_TIME_MODIFIED, 0, None)
            mtime = dateutil.parser.parse(
                finfo.get_modification_date_time().format_iso8601()
            )
            self.discover_file(f.get_path(), mtime)
        elif event_type == Gio.FileMonitorEvent.DELETED:
            self.delete_file(f)

    def discovery_done(self):
        self.numdiscoveries -= 1
        if not self.numdiscoveries:
            logger.debug("No more discoveries")
        self.check_quit()

    def dir_done(self):
        self.numdirs -= 1
        if not self.numdirs:
            logger.debug("No more dirs")
        self.check_quit()

    def check_quit(self):
        if not self.numdirs and not self.numdiscoveries and self.quit:
            self.stop()

    def file_stat(self, name, path, mtime):
        # Check if a file exists in the database
        f = self.get_file(name, path)
        if not f:
            return (False, True)
        # Make the mtime UTC otherwise the comparison will always fail
        fmtime = f.mtime.replace(tzinfo=timezone.utc)
        return (True, fmtime != mtime)

    def add_file(self, name, path, mtime):
        logger.debug("Adding file name={} path={} mtime={}".format(name, path, mtime))
        db_file = File(name=name, path=path, mtime=mtime)
        self.session.add(db_file)
        self.session.commit()
        return db_file

    def delete_file(self, f):
        logger.debug("File {} deleted".format(f.get_path()))
        (dirname, basename) = self.rel_path(f.get_path())
        db_file = self.get_file(basename, dirname)
        self.session.delete(db_file)
        self.session.commit()
        return db_file

    def get_file(self, name, path):
        db_file = self.session.query(File).filter_by(name=name, path=path).first()
        return db_file

    def store_stream_tags(self, db_stream, tags):
        def store_tag(tl, tag, *user_data):
            (copied, value) = Gst.TagList.copy_value(tl, tag)
            if copied:
                db_tag = Tag(stream=db_stream)
                db_tag.name = tag
                db_tag.value = Gst.value_serialize(value)
                self.session.add(db_tag)

        found = self._has_gst_override()
        if found:
            tags.foreach(store_tag, None)
        else:
            for (key, t, value) in self._parse_type_value(tags.to_string()):
                db_tag = Tag(stream=db_stream)
                db_tag.name = key
                db_tag.value = value
                self.session.add(db_tag)

    def store_structure(self, db_stream, s):
        def store_field(field_id, value, _unused):
            # Add every GstStructure field
            db_field = Field(stream=db_stream)
            db_field.name = GLib.quark_to_string(field_id)
            db_field.value = Gst.value_serialize(value)
            self.session.add(db_field)
            return True

        found = self._has_gst_override()
        if found:
            s.foreach(store_field, None)
        else:
            for (key, t, value) in self._parse_type_value(s.to_string()):
                db_field = Field(stream=db_stream)
                db_field.name = key
                db_field.value = value
                self.session.add(db_field)

    def store_stream_info(self, db_info, sinfo, parent=None):
        if not sinfo:
            return

        # Add the stream
        if sinfo.__gtype__.name == "GstDiscovererContainerInfo":
            db_stream = Container(info=db_info)
        elif sinfo.__gtype__.name == "GstDiscovererVideoInfo":
            db_stream = Video(info=db_info)
            db_stream.bitrate = sinfo.get_bitrate()
            db_stream.depth = sinfo.get_depth()
            db_stream.framerate_denom = sinfo.get_framerate_denom()
            db_stream.framerate_num = sinfo.get_framerate_num()
            db_stream.height = sinfo.get_height()
            db_stream.max_bitrate = sinfo.get_max_bitrate()
            db_stream.par_denom = sinfo.get_par_denom()
            db_stream.par_num = sinfo.get_par_num()
            db_stream.width = sinfo.get_width()
            db_stream.is_image = sinfo.is_image()
            db_stream.is_interlaced = sinfo.is_interlaced()
        elif sinfo.__gtype__.name == "GstDiscovererAudioInfo":
            db_stream = Audio(info=db_info)
            db_stream.channel_mask = sinfo.get_channel_mask()
            db_stream.channels = sinfo.get_channels()
            db_stream.sample_rate = sinfo.get_sample_rate()
            db_stream.bitrate = sinfo.get_bitrate()
            db_stream.max_bitrate = sinfo.get_max_bitrate()
            db_stream.sample_rate = sinfo.get_sample_rate()
            db_stream.depth = sinfo.get_depth()
            db_stream.language = sinfo.get_language()
        elif sinfo.__gtype__.name == "GstDiscovererSubtitleInfo":
            db_stream = Subtitle(info=db_info)
            db_stream.language = sinfo.get_language()
        elif sinfo.__gtype__.name == "GstDiscovererStreamInfo":
            db_stream = Stream(info=db_info)

        # Common fields
        if parent:
            parent.children.append(db_stream)
        s = sinfo.get_caps().get_structure(0)
        db_stream.media_type = s.get_name()
        self.session.add(db_stream)
        # Now the fields
        self.store_structure(db_stream, s)

        # Now the tags
        tags = sinfo.get_tags()
        if tags:
            self.store_stream_tags(db_stream, tags)

        next_sinfo = sinfo.get_next()
        if next_sinfo:
            self.store_stream_info(db_info, next_sinfo)
        elif sinfo.__gtype__.name == "GstDiscovererContainerInfo":
            for s in sinfo.get_streams():
                self.store_stream_info(db_info, s, db_stream)

    def on_discovered(self, discoverer, info, error):
        logger.debug("Discovered {}".format(info.get_uri()))
        if not info:
            self.discovery_done()
            return

        if error:
            logger.error("With error {}".format(error))

        path = urlparse(info.get_uri()).path
        (dirname, basename) = self.rel_path(path)

        result = info.get_result()
        db_file = self.get_file(basename, dirname)
        db_info = db_file.info
        if not db_info:
            db_info = Info(file=db_file)
            self.session.add(db_info)

        # Common properties
        db_info.duration = info.get_duration()
        db_info.live = info.get_live()
        db_info.seekable = info.get_seekable()
        # Number of streams
        db_info.audio_streams = len(info.get_audio_streams())
        db_info.video_streams = len(info.get_video_streams())
        db_info.subtitle_streams = len(info.get_subtitle_streams())

        # Topology
        # Remove every previous stream
        self.session.query(Stream).filter(Stream.info_id == db_info.id).delete()
        self.session.commit()

        sinfo = info.get_stream_info()
        self.store_stream_info(db_info, sinfo)

        # Finally commit
        self.session.commit()
        self.discovery_done()

    def on_finished(self, discoverer):
        logger.debug("Finished")

    def on_file_found(self, enum, res, user_data):
        files = enum.next_files_finish(res)
        if not files:
            self.dir_done()
            return

        for f in files:
            file_type = f.get_file_type()
            if file_type == Gio.FileType.DIRECTORY:
                # recurse
                path = os.path.join(enum.get_container().get_path(), f.get_name())
                subdir = Gio.File.new_for_path(path)
                logger.debug("Recursing {} (numdirs: {})".format(path, self.numdirs))
                self.numdirs += 1
                subdir.enumerate_children_async(
                    "*",
                    Gio.FileQueryInfoFlags.NOFOLLOW_SYMLINKS,
                    0,
                    None,
                    self.on_directory_content,
                    None,
                )
            elif file_type == Gio.FileType.REGULAR:
                path = os.path.join(enum.get_container().get_path(), f.get_name())
                mtime = dateutil.parser.parse(
                    f.get_modification_date_time().format_iso8601()
                )
                self.discover_file(path, mtime)

        enum.next_files_async(1, 0, None, self.on_file_found, None)

    def on_directory_content(self, path, res, user_data):
        enum = path.enumerate_children_finish(res)
        enum.next_files_async(1, 0, None, self.on_file_found, None)

    def start(self):
        self.loop.run()

    def stop(self):
        self.discoverer.stop()
        self.loop.quit()


def run():
    options = DiscovererOptions()
    args = options.parse_args()
    # Read the config file
    config = Config(args)
    discoverer = Discoverer(config, args)
    discoverer.start()



"""First commit

Revision ID: e827c1336bb4
Revises: 
Create Date: 2021-11-21 19:47:24.678501

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "e827c1336bb4"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "files",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=True),
        sa.Column("path", sa.String(), nullable=True),
        sa.Column("mtime", sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "metas",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("version", sa.String(), nullable=True),
        sa.Column("root", sa.String(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "infos",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("file_id", sa.Integer(), nullable=True),
        sa.Column("duration", sa.Integer(), nullable=True),
        sa.Column("seekable", sa.Boolean(), nullable=True),
        sa.Column("live", sa.Boolean(), nullable=True),
        sa.Column("audio_streams", sa.Integer(), nullable=True),
        sa.Column("video_streams", sa.Integer(), nullable=True),
        sa.Column("subtitle_streams", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ["file_id"],
            ["files.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "streams",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("parent_id", sa.Integer(), nullable=True),
        sa.Column("info_id", sa.Integer(), nullable=True),
        sa.Column("media_type", sa.String(), nullable=True),
        sa.Column("type", sa.String(), nullable=True),
        sa.Column("framerate_denom", sa.Integer(), nullable=True),
        sa.Column("framerate_num", sa.Integer(), nullable=True),
        sa.Column("height", sa.Integer(), nullable=True),
        sa.Column("par_denom", sa.Integer(), nullable=True),
        sa.Column("par_num", sa.Integer(), nullable=True),
        sa.Column("width", sa.Integer(), nullable=True),
        sa.Column("is_image", sa.Boolean(), nullable=True),
        sa.Column("is_interlaced", sa.Boolean(), nullable=True),
        sa.Column("bitrate", sa.Integer(), nullable=True),
        sa.Column("max_bitrate", sa.Integer(), nullable=True),
        sa.Column("depth", sa.Integer(), nullable=True),
        sa.Column("channel_mask", sa.Integer(), nullable=True),
        sa.Column("channels", sa.Integer(), nullable=True),
        sa.Column("sample_rate", sa.Integer(), nullable=True),
        sa.Column("language", sa.String(), nullable=True),
        sa.ForeignKeyConstraint(
            ["info_id"],
            ["infos.id"],
        ),
        sa.ForeignKeyConstraint(
            ["parent_id"],
            ["streams.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "fields",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("stream_id", sa.Integer(), nullable=True),
        sa.Column("name", sa.String(), nullable=True),
        sa.Column("value", sa.String(), nullable=True),
        sa.ForeignKeyConstraint(
            ["stream_id"],
            ["streams.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("fields")
    op.drop_table("streams")
    op.drop_table("infos")
    op.drop_table("metas")
    op.drop_table("files")
    # ### end Alembic commands ###



import os
from sqlite3 import Connection as SQLite3Connection

from alembic import command
from alembic.config import Config
from packaging.version import Version
from sqlalchemy import (
    BigInteger,
    Boolean,
    Column,
    DateTime,
    ForeignKey,
    Integer,
    MetaData,
    String,
    create_engine,
    event,
    inspect,
)
from sqlalchemy.engine import Engine
from sqlalchemy.orm import declarative_base, declared_attr, relationship, sessionmaker

naming_convention = {
    "ix": "ix_%(column_0_label)s",
    "uq": "uq_%(table_name)s_%(column_0_name)s",
    "ck": "ck_%(table_name)s_%(column_0_name)s",
    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    "pk": "pk_%(table_name)s",
}

Base = declarative_base()
Base.metadata = MetaData(naming_convention=naming_convention)


@event.listens_for(Engine, "connect")
def _set_sqlite_pragma(dbapi_connection, connection_record):
    if isinstance(dbapi_connection, SQLite3Connection):
        cursor = dbapi_connection.cursor()
        cursor.execute("PRAGMA foreign_keys=ON;")
        cursor.close()


class Schema(object):
    def __init__(self, config):
        self.config = config
        # Get the database configuration
        db_uri = config.get_database_uri()
        # Create, if needed, the database
        self.engine = create_engine(db_uri)
        self.sessionmaker = sessionmaker(bind=self.engine)
        self.migrate(db_uri)

    def migrate(self, db_uri):
        config = Config()
        cwd = os.path.abspath(os.path.dirname(__file__))
        config.set_main_option("script_location", os.path.join(cwd, "alembic"))
        config.set_main_option("sqlalchemy.url", db_uri)

        inspector = inspect(self.engine)
        tables = inspector.get_table_names()
        # Check if the table exists
        if "metas" in tables:
            session = self.create_session()
            meta = session.query(Meta).one_or_none()
            # Check the version in Meta
            # If version < 0.1.5, then stamp to e827c1336bb4
            if meta and Version(meta.version) < Version("0.1.5"):
                command.stamp(config, "e827c1336bb4")
        # Let alembic migrate to head, if needed
        command.upgrade(config, "head")

    def get_config(self):
        return self.config

    def create_session(self):
        return self.sessionmaker()


class Meta(Base):
    __tablename__ = "metas"
    id = Column(Integer, primary_key=True)
    version = Column(String)
    root = Column(String)


class File(Base):
    __tablename__ = "files"
    id = Column(Integer, primary_key=True, autoincrement=True, unique=True)
    name = Column(String)
    path = Column(String)
    mtime = Column(DateTime)

    info = relationship(
        "Info", uselist=False, back_populates="file", cascade="all, delete-orphan"
    )


class Info(Base):
    __tablename__ = "infos"
    id = Column(Integer, primary_key=True)
    file_id = Column(Integer, ForeignKey("files.id", ondelete="CASCADE"))

    duration = Column(BigInteger)
    seekable = Column(Boolean)
    live = Column(Boolean)
    audio_streams = Column(Integer)
    video_streams = Column(Integer)
    subtitle_streams = Column(Integer)

    file = relationship("File", back_populates="info")
    streams = relationship(
        "Stream", back_populates="info", cascade="all, delete-orphan"
    )


class Stream(Base):
    __tablename__ = "streams"
    id = Column(Integer, primary_key=True)
    parent_id = Column(Integer, ForeignKey("streams.id", ondelete="CASCADE"))

    info_id = Column(Integer, ForeignKey("infos.id", ondelete="CASCADE"))
    media_type = Column(String)
    type = Column(String)

    info = relationship("Info", back_populates="streams")
    fields = relationship(
        "Field", back_populates="stream", cascade="all, delete-orphan"
    )
    tags = relationship("Tag", back_populates="stream", cascade="all, delete-orphan")
    children = relationship("Stream", cascade="all, delete-orphan")

    __mapper_args__ = {"polymorphic_identity": "stream", "polymorphic_on": type}


class Video(Stream):
    framerate_denom = Column(Integer)
    framerate_num = Column(Integer)
    height = Column(Integer)
    par_denom = Column(Integer)
    par_num = Column(Integer)
    width = Column(Integer)
    is_image = Column(Boolean)
    is_interlaced = Column(Boolean)

    __mapper_args__ = {
        "polymorphic_identity": "video",
    }

    # Common columns found in other stream types
    @declared_attr
    def bitrate(cls):
        return Stream.__table__.c.get("bitrate", Column(Integer))

    @declared_attr
    def max_bitrate(cls):
        return Stream.__table__.c.get("max_bitrate", Column(Integer))

    @declared_attr
    def depth(cls):
        return Stream.__table__.c.get("depth", Column(Integer))


class Audio(Stream):
    channel_mask = Column(Integer)
    channels = Column(Integer)
    sample_rate = Column(Integer)

    __mapper_args__ = {
        "polymorphic_identity": "audio",
    }

    # Common columns found in other stream types
    @declared_attr
    def bitrate(cls):
        return Stream.__table__.c.get("bitrate", Column(Integer))

    @declared_attr
    def max_bitrate(cls):
        return Stream.__table__.c.get("max_bitrate", Column(Integer))

    @declared_attr
    def depth(cls):
        return Stream.__table__.c.get("depth", Column(Integer))

    @declared_attr
    def language(cls):
        return Stream.__table__.c.get("language", Column(String))


class Subtitle(Stream):
    __mapper_args__ = {
        "polymorphic_identity": "subtitle",
    }

    # Common columns found in other stream types
    @declared_attr
    def language(cls):
        return Stream.__table__.c.get("language", Column(String))


class Container(Stream):
    __mapper_args__ = {
        "polymorphic_identity": "container",
    }


class Field(Base):
    __tablename__ = "fields"
    id = Column(Integer, primary_key=True)

    stream_id = Column(Integer, ForeignKey("streams.id", ondelete="CASCADE"))

    name = Column(String)
    value = Column(String)

    stream = relationship("Stream", back_populates="fields")


class Tag(Base):
    __tablename__ = "tags"
    id = Column(Integer, primary_key=True)

    stream_id = Column(Integer, ForeignKey("streams.id", ondelete="CASCADE"))

    name = Column(String)
    value = Column(String)

    stream = relationship("Stream", back_populates="tags")



from argparse import ArgumentParser


class Options(ArgumentParser):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        group = self.add_argument_group("configuration file")
        group.add_argument("-c", "--config", action="store", help="configuration file")
        group = self.add_argument_group("database")
        group.add_argument("-i", "--uri", action="store", help="database uri")
        group.add_argument("-e", "--drivername", action="store", help="database driver")
        group.add_argument(
            "-u", "--username", action="store", help="database user name"
        )
        group.add_argument(
            "-p", "--password", action="store", help="database user password"
        )
        group.add_argument("-o", "--host", action="store", help="database host")
        group.add_argument("-r", "--port", action="store", help="database port")
        group.add_argument("-b", "--database", action="store", help="database")



"""empty message

Revision ID: 30374e2904cd
Revises: ad35ceceeb92
Create Date: 2022-02-03 09:22:27.872511

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "30374e2904cd"
down_revision = "ad35ceceeb92"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("infos", "duration", existing_type=sa.BigInteger(), nullable=True)

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("infos", "duration", existing_type=sa.INTEGER(), nullable=True)

    # ### end Alembic commands ###



import configparser
import os

from sqlalchemy.engine import URL, make_url


class ConfigError(Exception):
    pass


class Config(object):
    SYSTEM_CONFIG = "/etc/flumes"
    conf_database_args = [
        "uri",
        "drivername",
        "username",
        "password",
        "host",
        "port",
        "database",
    ]

    def __init__(self, args=None):
        user_configuration = "{}/{}".format(os.getenv("HOME"), ".flumes")
        self.config = configparser.ConfigParser()
        # try with the passed in configuration, if any
        if args and args.config:
            self.config.read(args.config)
        # try with the system wide configuration
        elif os.path.isfile(Config.SYSTEM_CONFIG):
            self.config.read(Config.SYSTEM_CONFIG)
        # try with the local user configuration
        elif os.path.isfile(user_configuration):
            self.config.read(user_configuration)

        # override now with passed in arguments
        if not "Database" in self.config:
            self.config["Database"] = {}

        if args:
            for c in self.conf_database_args:
                cv = getattr(args, c, None)
                if cv:
                    self.config["Database"][c] = str(cv)

        # Generate the other parameters based on the uri
        if "uri" in self.config["Database"]:
            url = make_url(self.config["Database"]["uri"])
            for c in self.conf_database_args:
                if c == "uri":
                    continue
                cv = getattr(url, c, None)
                if cv:
                    self.config["Database"][c] = str(cv)
                elif c in self.config["Database"]:
                    del self.config["Database"][c]

        if not "drivername" in self.config["Database"]:
            raise ConfigError("Missing drivername")

        # Generate the uri based on the other parameters
        url = URL.create(
            self.config["Database"]["drivername"],
            username=self.config["Database"].get("username", None),
            password=self.config["Database"].get("password", None),
            host=self.config["Database"].get("host", None),
            port=self.config["Database"].get("port", None),
            database=self.config["Database"].get("database", None),
        )
        self.config["Database"]["uri"] = str(url)

        if not "uri" in self.config["Database"]:
            raise ConfigError("Missing database uri")

    def get_database_uri(self):
        return self.config["Database"]["uri"]

    def get_database_driver(self):
        return self.config["Database"]["drivername"]

    def get_database_username(self):
        return self.config["Database"]["username"]

    def get_database_password(self):
        return self.config["Database"]["password"]

    def get_database_host(self):
        return self.config["Database"]["host"]

    def get_database_port(self):
        return self.config["Database"]["port"]

    def get_database_database(self):
        return self.config["Database"]["database"]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['flumes', 'flumes.alembic', 'flumes.alembic.versions']

package_data = \
{'': ['*']}

install_requires = \
['PyGObject>=3.42.0,<4.0.0',
 'SQLAlchemy>=1.4.26,<2.0.0',
 'alembic>=1.7.5,<2.0.0',
 'packaging>=21.3,<22.0',
 'python-dateutil>=2.8.2,<3.0.0']

entry_points = \
{'console_scripts': ['flumes-discovery = flumes.discoverer:run']}

setup_kwargs = {
    'name': 'flumes',
    'version': '0.1.13',
    'description': '',
    'long_description': None,
    'author': 'Jorge Zapata',
    'author_email': 'jorgeluis.zapata@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""empty message

Revision ID: ad35ceceeb92
Revises: a0b003abaac8
Create Date: 2021-11-22 13:35:37.574159

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "ad35ceceeb92"
down_revision = "a0b003abaac8"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "tags",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("stream_id", sa.Integer(), nullable=True),
        sa.Column("name", sa.String(), nullable=True),
        sa.Column("value", sa.String(), nullable=True),
        sa.ForeignKeyConstraint(
            ["stream_id"],
            ["streams.id"],
            name=op.f("fk_tags_stream_id_streams"),
            ondelete="CASCADE",
        ),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_tags")),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("tags")
    # ### end Alembic commands ###



"""empty message

Revision ID: a0b003abaac8
Revises: e827c1336bb4
Create Date: 2021-11-22 02:36:04.138227

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "a0b003abaac8"
down_revision = "e827c1336bb4"
branch_labels = None
depends_on = None

naming_convention = {
    "ix": "ix_%(column_0_label)s",
    "uq": "uq_%(table_name)s_%(column_0_name)s",
    "ck": "ck_%(table_name)s_%(column_0_name)s",
    "fk": "fk_%(table_name)s_%(column_0_name)s_%(referred_table_name)s",
    "pk": "pk_%(table_name)s",
}


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table(
        "fields", schema=None, naming_convention=naming_convention
    ) as batch_op:
        batch_op.drop_constraint("fk_fields_stream_id_streams", type_="foreignkey")
        batch_op.create_foreign_key(
            batch_op.f("fk_fields_stream_id_streams"),
            "streams",
            ["stream_id"],
            ["id"],
            ondelete="CASCADE",
        )

    with op.batch_alter_table(
        "infos", schema=None, naming_convention=naming_convention
    ) as batch_op:
        batch_op.drop_constraint("fk_infos_file_id_files", type_="foreignkey")
        batch_op.create_foreign_key(
            batch_op.f("fk_infos_file_id_files"),
            "files",
            ["file_id"],
            ["id"],
            ondelete="CASCADE",
        )

    with op.batch_alter_table(
        "streams", schema=None, naming_convention=naming_convention
    ) as batch_op:
        batch_op.drop_constraint("fk_streams_parent_id_streams", type_="foreignkey")
        batch_op.drop_constraint("fk_streams_info_id_infos", type_="foreignkey")
        batch_op.create_foreign_key(
            batch_op.f("fk_streams_parent_id_streams"),
            "streams",
            ["parent_id"],
            ["id"],
            ondelete="CASCADE",
        )
        batch_op.create_foreign_key(
            batch_op.f("fk_streams_info_id_infos"),
            "infos",
            ["info_id"],
            ["id"],
            ondelete="CASCADE",
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table(
        "streams", schema=None, naming_convention=naming_convention
    ) as batch_op:
        batch_op.drop_constraint(
            batch_op.f("fk_streams_info_id_infos"), type_="foreignkey"
        )
        batch_op.drop_constraint(
            batch_op.f("fk_streams_parent_id_streams"), type_="foreignkey"
        )
        batch_op.create_foreign_key(
            "fk_streams_parent_id_streams", "streams", ["parent_id"], ["id"]
        )
        batch_op.create_foreign_key(
            "fk_streams_info_id_infos", "infos", ["info_id"], ["id"]
        )

    with op.batch_alter_table(
        "infos", schema=None, naming_convention=naming_convention
    ) as batch_op:
        batch_op.drop_constraint(
            batch_op.f("fk_infos_file_id_files"), type_="foreignkey"
        )
        batch_op.create_foreign_key(
            "fk_infos_file_id_files", "files", ["file_id"], ["id"]
        )

    with op.batch_alter_table(
        "fields", schema=None, naming_convention=naming_convention
    ) as batch_op:
        batch_op.drop_constraint(
            batch_op.f("fk_fields_stream_id_streams"), type_="foreignkey"
        )
        batch_op.create_foreign_key(
            "fk_fields_stream_id_streams", "streams", ["stream_id"], ["id"]
        )

    # ### end Alembic commands ###
